@@ -104,8 +104,12 @@ static const ArgDesc argDesc[] = {
    "don't print any messages or errors"},
   {"-h",      argFlag,     &printHelp,     0,
    "print usage information"},
+  {"-?",      argFlag,     &printHelp,     0,
+   "print usage information"},
   {"-help",   argFlag,     &printHelp,     0,
    "print usage information"},
+  {"--help",  argFlag,     &printHelp,     0,
+   "print usage information"},
   {"-p",      argFlag,     &printHtml,     0,
    "exchange .pdf links by .html"}, 
   {"-c",      argFlag,     &complexMode,          0,
